@@ -57,11 +57,12 @@
 #include "igt_aux.h"
 #include "igt_debugfs.h"
 #include "igt_gt.h"
+#include "igt_rand.h"
 #include "config.h"
 #include "intel_reg.h"
 #include "ioctl_wrappers.h"
 #include "igt_kms.h"
-#include "igt_pm.h"
+#include "igt_stats.h"
 
 /**
  * SECTION:igt_aux
@@ -403,9 +404,6 @@ void igt_fork_hang_detector(int fd)
 {
 	struct stat st;
 
-	if (igt_only_list_subtests())
-		return;
-
 	igt_assert(fstat(fd, &st) == 0);
 
 	signal(SIGRTMAX, sig_abort);
@@ -415,9 +413,6 @@ void igt_fork_hang_detector(int fd)
 
 void igt_stop_hang_detector(void)
 {
-	if (igt_only_list_subtests())
-		return;
-
 	igt_stop_helper(&hang_detector);
 }
 #else
@@ -491,15 +486,6 @@ void igt_exchange_int(void *array, unsigned i, unsigned j)
 	int_arr[j] = tmp;
 }
 
-static uint32_t
-hars_petruska_f54_1_random_unsafe(void)
-{
-	static uint32_t state = 0x12345678;
-#define rol(x,k) ((x << k) | (x >> (32-k)))
-	return state = (state ^ rol (state, 5) ^ rol (state, 24)) + 0x37798849;
-#undef rol
-}
-
 /**
  * igt_permute_array:
  * @array: pointer to array
@@ -804,111 +790,6 @@ void igt_debug_manual_check(const char *var, const char *expected)
 	igt_assert(key != 'n' && key != 'N');
 }
 
-#define POWER_DIR "/sys/devices/pci0000:00/0000:00:02.0/power"
-/* We just leak this on exit ... */
-int pm_status_fd = -1;
-
-/**
- * igt_setup_runtime_pm:
- *
- * Sets up the runtime PM helper functions and enables runtime PM. To speed up
- * tests the autosuspend delay is set to 0.
- *
- * Returns:
- * True if runtime pm is available, false otherwise.
- */
-bool igt_setup_runtime_pm(void)
-{
-	int fd;
-	ssize_t size;
-	char buf[6];
-
-	if (pm_status_fd >= 0)
-		return true;
-
-	igt_pm_enable_audio_runtime_pm();
-
-	/* Our implementation uses autosuspend. Try to set it to 0ms so the test
-	 * suite goes faster and we have a higher probability of triggering race
-	 * conditions. */
-	fd = open(POWER_DIR "/autosuspend_delay_ms", O_WRONLY);
-	igt_assert_f(fd >= 0,
-		     "Can't open " POWER_DIR "/autosuspend_delay_ms\n");
-
-	/* If we fail to write to the file, it means this system doesn't support
-	 * runtime PM. */
-	size = write(fd, "0\n", 2);
-
-	close(fd);
-
-	if (size != 2)
-		return false;
-
-	/* We know we support runtime PM, let's try to enable it now. */
-	fd = open(POWER_DIR "/control", O_RDWR);
-	igt_assert_f(fd >= 0, "Can't open " POWER_DIR "/control\n");
-
-	size = write(fd, "auto\n", 5);
-	igt_assert(size == 5);
-
-	lseek(fd, 0, SEEK_SET);
-	size = read(fd, buf, ARRAY_SIZE(buf));
-	igt_assert(size == 5);
-	igt_assert(strncmp(buf, "auto\n", 5) == 0);
-
-	close(fd);
-
-	pm_status_fd = open(POWER_DIR "/runtime_status", O_RDONLY);
-	igt_assert_f(pm_status_fd >= 0,
-		     "Can't open " POWER_DIR "/runtime_status\n");
-
-	return true;
-}
-
-/**
- * igt_get_runtime_pm_status:
- *
- * Returns: The current runtime PM status.
- */
-enum igt_runtime_pm_status igt_get_runtime_pm_status(void)
-{
-	ssize_t n_read;
-	char buf[32];
-
-	lseek(pm_status_fd, 0, SEEK_SET);
-	n_read = read(pm_status_fd, buf, ARRAY_SIZE(buf));
-	igt_assert(n_read >= 0);
-	buf[n_read] = '\0';
-
-	if (strncmp(buf, "suspended\n", n_read) == 0)
-		return IGT_RUNTIME_PM_STATUS_SUSPENDED;
-	else if (strncmp(buf, "active\n", n_read) == 0)
-		return IGT_RUNTIME_PM_STATUS_ACTIVE;
-	else if (strncmp(buf, "suspending\n", n_read) == 0)
-		return IGT_RUNTIME_PM_STATUS_SUSPENDING;
-	else if (strncmp(buf, "resuming\n", n_read) == 0)
-		return IGT_RUNTIME_PM_STATUS_RESUMING;
-
-	igt_assert_f(false, "Unknown status %s\n", buf);
-	return IGT_RUNTIME_PM_STATUS_UNKNOWN;
-}
-
-/**
- * igt_wait_for_pm_status:
- * @status: desired runtime PM status
- *
- * Waits until for the driver to switch to into the desired runtime PM status,
- * with a 10 second timeout.
- *
- * Returns:
- * True if the desired runtime PM status was attained, false if the operation
- * timed out.
- */
-bool igt_wait_for_pm_status(enum igt_runtime_pm_status status)
-{
-	return igt_wait(igt_get_runtime_pm_status() == status, 10000, 100);
-}
-
 /* Functions with prefix kmstest_ independent of cairo library are pulled out
  * from file igt_kms.c since this file is skipped in lib/Android.mk when flag
  * ANDROID_HAS_CAIRO is 0. This ensures the usability of these functions even
@@ -1201,3 +1082,87 @@ void igt_set_module_param_int(const char *name, int val)
 
 	igt_set_module_param(name, str);
 }
+
+static struct igt_siglatency {
+	timer_t timer;
+	struct timespec target;
+	struct sigaction oldact;
+	struct igt_mean mean;
+
+	int sig;
+} igt_siglatency;
+
+static long delay(void)
+{
+	return hars_petruska_f54_1_random_unsafe() % (NSEC_PER_SEC / 1000);
+}
+
+static double elapsed(const struct timespec *now, const struct timespec *last)
+{
+	double nsecs;
+
+	nsecs = now->tv_nsec - last ->tv_nsec;
+	nsecs += 1e9*(now->tv_sec - last->tv_sec);
+
+	return nsecs;
+}
+
+static void siglatency(int sig, siginfo_t *info, void *arg)
+{
+	struct itimerspec its;
+
+	clock_gettime(CLOCK_MONOTONIC, &its.it_value);
+	if (info)
+		igt_mean_add(&igt_siglatency.mean,
+			     elapsed(&its.it_value, &igt_siglatency.target));
+	igt_siglatency.target = its.it_value;
+
+	its.it_value.tv_nsec += 100 * 1000;
+	its.it_value.tv_nsec += delay();
+	if (its.it_value.tv_nsec >= NSEC_PER_SEC) {
+		its.it_value.tv_nsec -= NSEC_PER_SEC;
+		its.it_value.tv_sec += 1;
+	}
+	its.it_interval.tv_sec = its.it_interval.tv_nsec = 0;
+	timer_settime(igt_siglatency.timer, TIMER_ABSTIME, &its, NULL);
+}
+
+void igt_start_siglatency(int sig)
+{
+	struct sigevent sev;
+	struct sigaction act;
+
+	if (sig <= 0)
+		sig = SIGRTMIN;
+
+	if (igt_siglatency.sig)
+		(void)igt_stop_siglatency(NULL);
+	igt_assert(igt_siglatency.sig == 0);
+	igt_siglatency.sig = sig;
+
+	memset(&sev, 0, sizeof(sev));
+	sev.sigev_notify = SIGEV_SIGNAL | SIGEV_THREAD_ID;
+	sev.sigev_notify_thread_id = gettid();
+	sev.sigev_signo = sig;
+	timer_create(CLOCK_MONOTONIC, &sev, &igt_siglatency.timer);
+
+	memset(&act, 0, sizeof(act));
+	act.sa_sigaction = siglatency;
+	sigaction(sig, &act, &igt_siglatency.oldact);
+
+	siglatency(sig, NULL, NULL);
+}
+
+double igt_stop_siglatency(struct igt_mean *result)
+{
+	double mean = igt_mean_get(&igt_siglatency.mean);
+
+	if (result)
+		*result = igt_siglatency.mean;
+
+	sigaction(igt_siglatency.sig, &igt_siglatency.oldact, NULL);
+	timer_delete(igt_siglatency.timer);
+	memset(&igt_siglatency, 0, sizeof(igt_siglatency));
+
+	return mean;
+}
