@@ -32,6 +32,7 @@
 
 #include <setjmp.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -106,7 +107,7 @@ void __igt_fixture_end(void) __attribute__((noreturn));
  * enumeration (e.g. when enumerating on systems without an intel gpu) such
  * blocks should be annotated with igt_fixture.
  */
-#define igt_fixture for (int igt_tokencat(__tmpint,__LINE__) = 0; \
+#define igt_fixture for (volatile int igt_tokencat(__tmpint,__LINE__) = 0; \
 			 igt_tokencat(__tmpint,__LINE__) < 1 && \
 			 __igt_fixture() && \
 			 (sigsetjmp(igt_subtest_jmpbuf, 1) == 0); \
@@ -196,6 +197,30 @@ bool __igt_run_subtest(const char *subtest_name);
 const char *igt_subtest_name(void);
 bool igt_only_list_subtests(void);
 
+void __igt_subtest_group_save(int *);
+void __igt_subtest_group_restore(int);
+/**
+ * igt_subtest_group:
+ *
+ * Group a set of subtests together with their common setup code
+ *
+ * Testcase with subtests often need to set up a bunch of shared state as the
+ * common test fixture. But if there are multiple with different requirements
+ * the commont setup code can't be extracted, since a test condition failure in
+ * e.g. igt_require() would result in all subsequent tests skipping. Even those
+ * from a different group.
+ *
+ * This macro allows to group together a set of #igt_fixture and #igt_subtest
+ * clauses. If any common setup in a fixture fails, only the subtests in this
+ * group will fail or skip. Subtest groups can be arbitrarily nested.
+ */
+#define igt_subtest_group for (int igt_tokencat(__tmpint,__LINE__) = 0, \
+			       igt_tokencat(__save,__LINE__) = 0; \
+			       igt_tokencat(__tmpint,__LINE__) < 1 && \
+			       (__igt_subtest_group_save(& igt_tokencat(__save,__LINE__) ), true); \
+			       igt_tokencat(__tmpint,__LINE__) ++, \
+			       __igt_subtest_group_restore(igt_tokencat(__save,__LINE__) ))
+
 /**
  * igt_main:
  *
@@ -271,6 +296,17 @@ void __igt_fail_assert(const char *domain, const char *file,
 	__attribute__((noreturn));
 void igt_exit(void) __attribute__((noreturn));
 
+/**
+ * igt_ignore_warn:
+ * @expr: condition to ignore
+ *
+ *
+ * Stops the compiler warning about an unused return value.
+ */
+static inline void igt_ignore_warn(bool value)
+{
+}
+
 /**
  * igt_assert:
  * @expr: condition to test
@@ -667,8 +703,6 @@ typedef void (*igt_exit_handler_t)(int sig);
 
 /* reliable atexit helpers, also work when killed by a signal (if possible) */
 void igt_install_exit_handler(igt_exit_handler_t fn);
-void igt_enable_exit_handler(void);
-void igt_disable_exit_handler(void);
 
 /* helpers to automatically reduce test runtime in simulation */
 bool igt_run_in_simulation(void);
@@ -686,7 +720,16 @@ void igt_skip_on_simulation(void);
 
 extern const char *igt_interactive_debug;
 
-/* structured logging */
+/**
+ * igt_log_level:
+ * @IGT_LOG_DEBUG: debug information, not printed by default
+ * @IGT_LOG_INFO: informational message, printed by default
+ * @IGT_LOG_WARN: non-fatal warnings which should be treated as test failures
+ * @IGT_LOG_CRITICAL: critical errors which lead to immediate termination of tests
+ * @IGT_LOG_NONE: unused
+ *
+ * Log levels used by functions like igt_log().
+ */
 enum igt_log_level {
 	IGT_LOG_DEBUG,
 	IGT_LOG_INFO,
@@ -777,6 +820,40 @@ extern enum igt_log_level igt_log_level;
 void igt_set_timeout(unsigned int seconds,
 		     const char *op);
 
+/**
+ * igt_nsec_elapsed:
+ * @start: measure from this point in time
+ *
+ * Reports the difference in the monotonic clock from the start time
+ * in nanoseconds. On the first invocation, start should be zeroed and will
+ * be set by the call.
+ *
+ * Typical use would be:
+ *
+ * igt_subtest("test") {
+ * 	struct timespec start = {};
+ * 	while (igt_nsec_elapsed(&start) < test_timeout_ns)
+ *	 	do_test();
+ * }
+ *
+ * A handy approximation is to use nsec >> 30 to convert to seconds,
+ * nsec >> 20 to convert to milliseconds - the error is about 8%, acceptable
+ * for test run times.
+ */
+uint64_t igt_nsec_elapsed(struct timespec *start);
+
+/**
+ * igt_seconds_elapsed:
+ * @start: measure from this point in time
+ *
+ * A wrapper around igt_nsec_elapsed that reports the approximate (8% error)
+ * number of seconds since the start point.
+ */
+static inline uint32_t igt_seconds_elapsed(struct timespec *start)
+{
+	return igt_nsec_elapsed(start) >> 30;
+}
+
 void igt_reset_timeout(void);
 
 FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
@@ -785,8 +862,8 @@ FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
  * igt_fopen_data:
  * @filename: filename to open.
  *
- * Open a datafile for test, first try from installation directory
- * then from build directory.
+ * Open a datafile for test, first try from installation directory,
+ * then from build directory, and finally from current directory.
  */
 #define igt_fopen_data(filename) \
 	__igt_fopen_data(IGT_SRCDIR, IGT_DATADIR, filename)
