@@ -47,7 +47,7 @@
  *
  * This library provides helpers to access debugfs features. On top of some
  * basic functions to access debugfs files with e.g. igt_debugfs_open() it also
- * provides higher-level wrappers for some debugfs features
+ * provides higher-level wrappers for some debugfs features.
  *
  * # Pipe CRC Support
  *
@@ -87,28 +87,29 @@ typedef struct {
 	char dri_path[128];
 } igt_debugfs_t;
 
+static const char *__debugfs_mount(void)
+{
+	struct stat st;
+
+	if (stat("/debug/dri", &st) == 0)
+		return "/debug";
+
+	if (stat("/sys/kernel/debug/dri", &st) == 0)
+		return "/sys/kernel/debug";
+
+	igt_assert(stat("/sys/kernel/debug", &st) == 0);
+	igt_assert(mount("debug", "/sys/kernel/debug", "debugfs", 0, 0) == 0);
+	return "/sys/kernel/debug";
+}
+
 static bool __igt_debugfs_init(igt_debugfs_t *debugfs)
 {
-	const char *path = "/sys/kernel/debug";
 	struct stat st;
 	int n;
 
-	if (stat("/debug/dri", &st) == 0) {
-		path = "/debug/dri";
-		goto find_minor;
-	}
-
-	if (stat("/sys/kernel/debug/dri", &st) == 0)
-		goto find_minor;
-
-	igt_assert(stat("/sys/kernel/debug", &st) == 0);
-
-	mount("debug", "/sys/kernel/debug", "debugfs", 0, 0);
-
-find_minor:
-	strcpy(debugfs->root, path);
+	strcpy(debugfs->root, __debugfs_mount());
 	for (n = 0; n < 16; n++) {
-		int len = sprintf(debugfs->dri_path, "%s/dri/%d", path, n);
+		int len = sprintf(debugfs->dri_path, "%s/dri/%d", debugfs->root, n);
 		sprintf(debugfs->dri_path + len, "/i915_error_state");
 		if (stat(debugfs->dri_path, &st) == 0) {
 			debugfs->dri_path[len] = '\0';
@@ -117,7 +118,6 @@ find_minor:
 	}
 
 	debugfs->dri_path[0] = '\0';
-
 	return false;
 }
 
@@ -550,15 +550,19 @@ void igt_pipe_crc_stop(igt_pipe_crc_t *pipe_crc)
  * @n_crcs: number of CRCs to capture
  * @out_crcs: buffer pointer for the captured CRC values
  *
- * Read @n_crcs from @pipe_crc. This function blocks until @n_crcs are
- * retrieved. @out_crcs is alloced by this function and must be released with
- * free() by the caller.
+ * Read up to @n_crcs from @pipe_crc. This function does not block, and will
+ * return early if not enough CRCs can be captured, if @pipe_crc has been
+ * opened using igt_pipe_crc_new_nonblock(). It will block until @n_crcs are
+ * retrieved if @pipe_crc has been opened using igt_pipe_crc_new(). @out_crcs is
+ * alloced by this function and must be released with free() by the caller.
  *
  * Callers must start and stop the capturing themselves by calling
- * igt_pipe_crc_start() and igt_pipe_crc_stop().
+ * igt_pipe_crc_start() and igt_pipe_crc_stop(). For one-shot CRC collecting
+ * look at igt_pipe_crc_collect_crc().
  *
- * Returns: The number of CRCs captured. Should be equal to @n_crcs in blocking
- * mode, but can be less (even zero) in non-blocking mode.
+ * Returns:
+ * The number of CRCs captured. Should be equal to @n_crcs in blocking mode, but
+ * can be less (even zero) in non-blocking mode.
  */
 int
 igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
@@ -609,13 +613,18 @@ static void crc_sanity_checks(igt_crc_t *crc)
  * @out_crc: buffer for the captured CRC values
  *
  * Read a single CRC from @pipe_crc. This function blocks until the CRC is
- * retrieved.  @out_crc must be allocated by the caller.
+ * retrieved, irrespective of whether @pipe_crc has been opened with
+ * igt_pipe_crc_new() or igt_pipe_crc_new_nonblock().  @out_crc must be
+ * allocated by the caller.
  *
  * This function takes care of the pipe_crc book-keeping, it will start/stop
  * the collection of the CRC.
  *
  * This function also calls the interactive debug with the "crc" domain, so you
  * can make use of this feature to actually see the screen that is being CRC'd.
+ *
+ * For continuous CRC collection look at igt_pipe_crc_start(),
+ * igt_pipe_crc_get_crcs() and igt_pipe_crc_stop().
  */
 void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
 {
@@ -770,3 +779,28 @@ int igt_get_stable_obj_count(int driver)
 	return obj_count;
 }
 
+
+/* Non-i915 specific debugfs API */
+
+/**
+ * igt_debugfs_dir:
+ * @device: fd of the device (or -1 to default to Intel)
+ *
+ * This opens the debugfs directory corresponding to device for use
+ * with igt_sysfs_get() and related functions.
+ *
+ * Returns:
+ * The directory fd, or -1 on failure.
+ */
+int igt_debugfs_dir(int device)
+{
+	struct stat st;
+	char path[256];
+
+	if (fstat(device, &st) || !S_ISCHR(st.st_mode))
+		return -1;
+
+	sprintf(path, "%s/dri/%d", __debugfs_mount(), (int)(st.st_rdev & 0xff));
+	igt_debug("Opening debugfs dir %s\n", path);
+	return open(path, O_RDONLY);
+}
