@@ -124,7 +124,6 @@ __attribute__((warn_unused_result))
 int igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
 			  igt_crc_t **out_crcs);
 void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc);
-bool igt_crc_equal(igt_crc_t *a, igt_crc_t *b);
 
 /*
  * Drop caches
@@ -155,6 +154,11 @@ bool igt_crc_equal(igt_crc_t *a, igt_crc_t *b);
  * Also drop active objects once retired.
  */
 #define DROP_ACTIVE 0x8
+/**
+ * DROP_ALL:
+ *
+ * All of the above DROP_ flags combined.
+ */
 #define DROP_ALL (DROP_UNBOUND | \
 		  DROP_BOUND | \
 		  DROP_RETIRE | \
@@ -175,4 +179,6 @@ void igt_enable_prefault(void);
  */
 int igt_get_stable_obj_count(int driver);
 
+int igt_debugfs_dir(int device);
+
 #endif /* __IGT_DEBUGFS_H__ */
