@@ -31,6 +31,7 @@
 
 #include "drmtest.h"
 #include "igt_fb.h"
+#include "igt_kms.h"
 #include "ioctl_wrappers.h"
 #include "intel_chipset.h"
 
@@ -76,15 +77,14 @@ static struct format_desc_struct {
 static void igt_get_fb_tile_size(int fd, uint64_t tiling, int fb_bpp,
 				 unsigned *width_ret, unsigned *height_ret)
 {
-	uint32_t devid = intel_get_drm_devid(fd);
-
 	switch (tiling) {
 	case LOCAL_DRM_FORMAT_MOD_NONE:
 		*width_ret = 64;
 		*height_ret = 1;
 		break;
 	case LOCAL_I915_FORMAT_MOD_X_TILED:
-		if (intel_gen(devid) == 2) {
+		igt_require_intel(fd);
+		if (intel_gen(intel_get_drm_devid(fd)) == 2) {
 			*width_ret = 128;
 			*height_ret = 16;
 		} else {
@@ -93,13 +93,20 @@ static void igt_get_fb_tile_size(int fd, uint64_t tiling, int fb_bpp,
 		}
 		break;
 	case LOCAL_I915_FORMAT_MOD_Y_TILED:
-		if (IS_915(devid))
-			*width_ret = 512;
-		else
+		igt_require_intel(fd);
+		if (intel_gen(intel_get_drm_devid(fd)) == 2) {
 			*width_ret = 128;
-		*height_ret = 32;
+			*height_ret = 16;
+		} else if (IS_915(intel_get_drm_devid(fd))) {
+			*width_ret = 512;
+			*height_ret = 8;
+		} else {
+			*width_ret = 128;
+			*height_ret = 32;
+		}
 		break;
 	case LOCAL_I915_FORMAT_MOD_Yf_TILED:
+		igt_require_intel(fd);
 		switch (fb_bpp) {
 		case 8:
 			*width_ret = 64;
@@ -145,8 +152,8 @@ void igt_calc_fb_size(int fd, int width, int height, int bpp, uint64_t tiling,
 
 	igt_get_fb_tile_size(fd, tiling, bpp, &tile_width, &tile_height);
 
-	if (intel_gen(intel_get_drm_devid(fd)) <= 3 &&
-	    tiling != LOCAL_DRM_FORMAT_MOD_NONE) {
+	if (tiling != LOCAL_DRM_FORMAT_MOD_NONE &&
+	    intel_gen(intel_get_drm_devid(fd)) <= 3) {
 		int v;
 
 		/* Round the tiling up to the next power-of-two and the region
@@ -174,32 +181,85 @@ void igt_calc_fb_size(int fd, int width, int height, int bpp, uint64_t tiling,
 }
 
 /* helpers to create nice-looking framebuffers */
-static int create_bo_for_fb(int fd, int width, int height, int bpp,
-			    uint64_t tiling, unsigned bo_size,
-			    unsigned bo_stride, uint32_t *gem_handle_ret,
-			    unsigned *size_ret, unsigned *stride_ret)
+static int create_bo_for_fb(int fd, int width, int height, uint32_t format,
+			    uint64_t tiling, unsigned size, unsigned stride,
+			    unsigned *size_ret, unsigned *stride_ret,
+			    bool *is_dumb)
 {
-	uint32_t gem_handle;
-	int ret = 0;
-	unsigned size, stride;
+	int bpp = igt_drm_format_to_bpp(format);
+	int bo;
 
-	igt_calc_fb_size(fd, width, height, bpp, tiling, &size, &stride);
-	if (bo_size == 0)
-		bo_size = size;
-	if (bo_stride == 0)
-		bo_stride = stride;
+	if (tiling || size || stride) {
+		unsigned calculated_size, calculated_stride;
 
-	gem_handle = gem_create(fd, bo_size);
+		igt_calc_fb_size(fd, width, height, bpp, tiling,
+				 &calculated_size, &calculated_stride);
+		if (stride == 0)
+			stride = calculated_stride;
+		if (size == 0)
+			size = calculated_size;
 
-	if (tiling == LOCAL_I915_FORMAT_MOD_X_TILED)
-		ret = __gem_set_tiling(fd, gem_handle, I915_TILING_X,
-				       bo_stride);
+		if (is_dumb)
+			*is_dumb = false;
 
-	*stride_ret = bo_stride;
-	*size_ret = bo_size;
-	*gem_handle_ret = gem_handle;
+		if (is_i915_device(fd)) {
+			uint32_t *ptr;
 
-	return ret;
+			bo = gem_create(fd, size);
+			gem_set_tiling(fd, bo, tiling, stride);
+
+			/* Ensure the framebuffer is preallocated */
+			ptr = gem_mmap__gtt(fd, bo, size, PROT_READ);
+			igt_assert(*ptr == 0);
+			munmap(ptr, size);
+
+			if (size_ret)
+				*size_ret = size;
+
+			if (stride_ret)
+				*stride_ret = stride;
+
+			return bo;
+		} else {
+			bool driver_has_gem_api = false;
+
+			igt_require(driver_has_gem_api);
+			return -EINVAL;
+		}
+	} else {
+		if (is_dumb)
+			*is_dumb = true;
+
+		return kmstest_dumb_create(fd, width, height, bpp, stride_ret,
+					   size_ret);
+	}
+}
+
+/**
+ * igt_create_bo_with_dimensions:
+ * @fd: open drm file descriptor
+ * @width: width of the buffer object in pixels
+ * @height: height of the buffer object in pixels
+ * @format: drm fourcc pixel format code
+ * @modifier: modifier corresponding to the tiling layout of the buffer object
+ * @stride: stride of the buffer object in bytes (0 for automatic stride)
+ * @size_ret: size of the buffer object as created by the kernel
+ * @stride_ret: stride of the buffer object as created by the kernel
+ * @is_dumb: whether the created buffer object is a dumb buffer or not
+ *
+ * This function allocates a gem buffer object matching the requested
+ * properties.
+ *
+ * Returns:
+ * The kms id of the created buffer object.
+ */
+int igt_create_bo_with_dimensions(int fd, int width, int height,
+				  uint32_t format, uint64_t modifier,
+				  unsigned stride, unsigned *size_ret,
+				  unsigned *stride_ret, bool *is_dumb)
+{
+	return create_bo_for_fb(fd, width, height, format, modifier, 0, stride,
+			     size_ret, stride_ret, is_dumb);
 }
 
 /**
@@ -277,6 +337,40 @@ igt_paint_color_gradient(cairo_t *cr, int x, int y, int w, int h,
 	cairo_pattern_destroy(pat);
 }
 
+/**
+ * igt_paint_color_gradient_range:
+ * @cr: cairo drawing context
+ * @x: pixel x-coordination of the fill rectangle
+ * @y: pixel y-coordination of the fill rectangle
+ * @w: width of the fill rectangle
+ * @h: height of the fill rectangle
+ * @sr: red value to use as start gradient color
+ * @sg: green value to use as start gradient color
+ * @sb: blue value to use as start gradient color
+ * @er: red value to use as end gradient color
+ * @eg: green value to use as end gradient color
+ * @eb: blue value to use as end gradient color
+ *
+ * This functions draws a gradient into the rectangle which fades in
+ * from one color to the other using the drawing context @cr.
+ */
+void
+igt_paint_color_gradient_range(cairo_t *cr, int x, int y, int w, int h,
+			       double sr, double sg, double sb,
+			       double er, double eg, double eb)
+{
+	cairo_pattern_t *pat;
+
+	pat = cairo_pattern_create_linear(x, y, x + w, y + h);
+	cairo_pattern_add_color_stop_rgba(pat, 1, sr, sg, sb, 1);
+	cairo_pattern_add_color_stop_rgba(pat, 0, er, eg, eb, 1);
+
+	cairo_rectangle(cr, x, y, w, h);
+	cairo_set_source(cr, pat);
+	cairo_fill(cr);
+	cairo_pattern_destroy(pat);
+}
+
 static void
 paint_test_patterns(cairo_t *cr, int width, int height)
 {
@@ -500,17 +594,15 @@ igt_create_fb_with_bo_size(int fd, int width, int height,
 			   unsigned bo_stride)
 {
 	uint32_t fb_id;
-	int bpp;
 
 	memset(fb, 0, sizeof(*fb));
 
-	bpp = igt_drm_format_to_bpp(format);
-
-	igt_debug("%s(width=%d, height=%d, format=0x%x [bpp=%d], tiling=0x%"PRIx64", size=%d)\n",
-		  __func__, width, height, format, bpp, tiling, bo_size);
-	do_or_die(create_bo_for_fb(fd, width, height, bpp, tiling, bo_size,
-				   bo_stride, &fb->gem_handle, &fb->size,
-				   &fb->stride));
+	igt_debug("%s(width=%d, height=%d, format=0x%x, tiling=0x%"PRIx64", size=%d)\n",
+		  __func__, width, height, format, tiling, bo_size);
+	fb->gem_handle = create_bo_for_fb(fd, width, height, format, tiling,
+					  bo_size, bo_stride, &fb->size,
+					  &fb->stride, &fb->is_dumb);
+	igt_assert(fb->gem_handle > 0);
 
 	igt_debug("%s(handle=%d, pitch=%d)\n",
 		  __func__, fb->gem_handle, fb->stride);
@@ -542,6 +634,7 @@ igt_create_fb_with_bo_size(int fd, int width, int height,
 	fb->tiling = tiling;
 	fb->drm_format = format;
 	fb->fb_id = fb_id;
+	fb->fd = fd;
 
 	return fb_id;
 }
@@ -859,6 +952,7 @@ struct fb_blit_upload {
 		uint32_t handle;
 		unsigned size, stride;
 		uint8_t *map;
+		bool is_dumb;
 	} linear;
 };
 
@@ -912,7 +1006,6 @@ static void create_cairo_surface__blit(int fd, struct igt_fb *fb)
 	struct fb_blit_upload *blit;
 	cairo_format_t cairo_format;
 	unsigned int obj_tiling = fb_mod_to_obj_tiling(fb->tiling);
-	int bpp, ret;
 
 	blit = malloc(sizeof(*blit));
 	igt_assert(blit);
@@ -922,14 +1015,14 @@ static void create_cairo_surface__blit(int fd, struct igt_fb *fb)
 	 * cairo). This linear bo will be then blitted to its final
 	 * destination, tiling it at the same time.
 	 */
-	bpp = igt_drm_format_to_bpp(fb->drm_format);
-	ret = create_bo_for_fb(fd, fb->width, fb->height, bpp,
-				LOCAL_DRM_FORMAT_MOD_NONE, 0, 0,
-				&blit->linear.handle,
-				&blit->linear.size,
-				&blit->linear.stride);
+	blit->linear.handle = create_bo_for_fb(fd, fb->width, fb->height,
+					       fb->drm_format,
+					       LOCAL_DRM_FORMAT_MOD_NONE, 0,
+					       0, &blit->linear.size,
+					       &blit->linear.stride,
+					       &blit->linear.is_dumb);
 
-	igt_assert(ret == 0);
+	igt_assert(blit->linear.handle > 0);
 
 	blit->fd = fd;
 	blit->fb = fb;
@@ -967,28 +1060,54 @@ static void create_cairo_surface__blit(int fd, struct igt_fb *fb)
 						    cairo_format,
 						    fb->width, fb->height,
 						    blit->linear.stride);
+	fb->domain = I915_GEM_DOMAIN_GTT;
 
 	cairo_surface_set_user_data(fb->cairo_surface,
 				    (cairo_user_data_key_t *)create_cairo_surface__blit,
 				    blit, destroy_cairo_surface__blit);
 }
 
+/**
+ * igt_dirty_fb:
+ * @fd: open drm file descriptor
+ * @fb: pointer to an #igt_fb structure
+ *
+ * Flushes out the whole framebuffer.
+ *
+ * Returns: 0 upon success.
+ */
+int igt_dirty_fb(int fd, struct igt_fb *fb)
+{
+	return drmModeDirtyFB(fb->fd, fb->fb_id, NULL, 0);
+}
+
 static void destroy_cairo_surface__gtt(void *arg)
 {
 	struct igt_fb *fb = arg;
 
 	munmap(cairo_image_surface_get_data(fb->cairo_surface), fb->size);
 	fb->cairo_surface = NULL;
+
+	if (fb->is_dumb)
+		igt_dirty_fb(fb->fd, fb);
 }
 
 static void create_cairo_surface__gtt(int fd, struct igt_fb *fb)
 {
-	void *ptr = gem_mmap__gtt(fd, fb->gem_handle, fb->size, PROT_READ | PROT_WRITE);
+	void *ptr;
+
+	if (fb->is_dumb)
+		ptr = kmstest_dumb_map_buffer(fd, fb->gem_handle, fb->size,
+					      PROT_READ | PROT_WRITE);
+	else
+		ptr = gem_mmap__gtt(fd, fb->gem_handle, fb->size,
+				    PROT_READ | PROT_WRITE);
 
 	fb->cairo_surface =
 		cairo_image_surface_create_for_data(ptr,
 						    drm_format_to_cairo(fb->drm_format),
 						    fb->width, fb->height, fb->stride);
+	fb->domain = I915_GEM_DOMAIN_GTT;
 
 	cairo_surface_set_user_data(fb->cairo_surface,
 				    (cairo_user_data_key_t *)create_cairo_surface__gtt,
@@ -1005,8 +1124,9 @@ static cairo_surface_t *get_cairo_surface(int fd, struct igt_fb *fb)
 			create_cairo_surface__gtt(fd, fb);
 	}
 
-	gem_set_domain(fd, fb->gem_handle,
-		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
+	if (!fb->is_dumb)
+		gem_set_domain(fd, fb->gem_handle, I915_GEM_DOMAIN_CPU,
+			       I915_GEM_DOMAIN_CPU);
 
 	igt_assert(cairo_surface_status(fb->cairo_surface) == CAIRO_STATUS_SUCCESS);
 	return fb->cairo_surface;
