@@ -1,38 +1,7 @@
 #!/bin/sh
 
-IGT_EXIT_TIMEOUT=78
-IGT_EXIT_SKIP=77
-IGT_EXIT_SUCCESS=0
-IGT_EXIT_INVALID=79
-IGT_EXIT_FAILURE=99
-
-# hacked-up long option parsing
-for arg in $@ ; do
-	case $arg in
-		--list-subtests)
-			exit $IGT_EXIT_INVALID
-			;;
-		--run-subtest)
-			exit $IGT_EXIT_INVALID
-			;;
-		--debug)
-			IGT_LOG_LEVEL=debug
-			;;
-		--help-description)
-			echo $IGT_TEST_DESCRIPTION
-			exit $IGT_EXIT_SUCCESS
-			;;
-		--help)
-			echo "Usage: `basename $0` [OPTIONS]"
-			echo "  --list-subtests"
-			echo "  --run-subtest <pattern>"
-			echo "  --debug"
-			echo "  --help-description"
-			echo "  --help"
-			exit $IGT_EXIT_SUCCESS
-			;;
-	esac
-done
+SOURCE_DIR="$( dirname "${BASH_SOURCE[0]}" )"
+. $SOURCE_DIR/drm_getopt.sh
 
 skip() {
 	echo "$@"
