@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -37,10 +38,6 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
 
 #define OBJECT_SIZE 16384
 
@@ -50,6 +47,9 @@
 #define BLT_SRC_TILED		(1<<15)
 #define BLT_DST_TILED		(1<<11)
 
+#define LOCAL_I915_EXEC_NO_RELOC (1<<11)
+#define LOCAL_I915_EXEC_HANDLE_LUT (1<<12)
+
 static int gem_linear_blt(int fd,
 			  uint32_t *batch,
 			  uint32_t src,
@@ -60,7 +60,7 @@ static int gem_linear_blt(int fd,
 	uint32_t *b = batch;
 	int height = length / (16 * 1024);
 
-	igt_assert(height <= 1<<16);
+	igt_assert_lte(height, 1 << 16);
 
 	if (height) {
 		int i = 0;
@@ -170,7 +170,18 @@ static const char *bytes_per_sec(char *buf, double v)
 	return buf;
 }
 
-static void run(int object_size)
+static int dcmp(const void *A, const void *B)
+{
+	const double *a = A, *b = B;
+	if (*a < *b)
+		return -1;
+	else if (*a > *b)
+		return 1;
+	else
+		return 0;
+}
+
+static void run(int object_size, bool dumb)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 exec[3];
@@ -180,31 +191,21 @@ static void run(int object_size)
 	int fd, len, count;
 	int ring;
 
-	fd = drm_open_any();
-	handle = gem_create(fd, 4096);
+	fd = drm_open_driver(DRIVER_INTEL);
+	if (dumb)
+		handle = kmstest_dumb_create(fd, 32, 32, 32, NULL, NULL);
+	else
+		handle = gem_create(fd, 4096);
+
 	src = gem_create(fd, object_size);
 	dst = gem_create(fd, object_size);
 
-	len = gem_linear_blt(fd, buf, src, dst, object_size, reloc);
+	len = gem_linear_blt(fd, buf, 0, 1, object_size, reloc);
 	gem_write(fd, handle, 0, buf, len);
 
+	memset(exec, 0, sizeof(exec));
 	exec[0].handle = src;
-	exec[0].relocation_count = 0;
-	exec[0].relocs_ptr = 0;
-	exec[0].alignment = 0;
-	exec[0].offset = 0;
-	exec[0].flags = 0;
-	exec[0].rsvd1 = 0;
-	exec[0].rsvd2 = 0;
-
 	exec[1].handle = dst;
-	exec[1].relocation_count = 0;
-	exec[1].relocs_ptr = 0;
-	exec[1].alignment = 0;
-	exec[1].offset = 0;
-	exec[1].flags = 0;
-	exec[1].rsvd1 = 0;
-	exec[1].rsvd2 = 0;
 
 	exec[2].handle = handle;
 	if (intel_gen(intel_get_drm_devid(fd)) >= 8)
@@ -212,40 +213,50 @@ static void run(int object_size)
 	else
 		exec[2].relocation_count = len > 40 ? 4 : 2;
 	exec[2].relocs_ptr = (uintptr_t)reloc;
-	exec[2].alignment = 0;
-	exec[2].offset = 0;
-	exec[2].flags = 0;
-	exec[2].rsvd1 = 0;
-	exec[2].rsvd2 = 0;
 
 	ring = 0;
 	if (HAS_BLT_RING(intel_get_drm_devid(fd)))
 		ring = I915_EXEC_BLT;
 
+	memset(&execbuf, 0, sizeof(execbuf));
 	execbuf.buffers_ptr = (uintptr_t)exec;
 	execbuf.buffer_count = 3;
-	execbuf.batch_start_offset = 0;
 	execbuf.batch_len = len;
-	execbuf.cliprects_ptr = 0;
-	execbuf.num_cliprects = 0;
-	execbuf.DR1 = 0;
-	execbuf.DR4 = 0;
 	execbuf.flags = ring;
-	i915_execbuffer2_set_context_id(execbuf, 0);
-	execbuf.rsvd2 = 0;
+	execbuf.flags |= LOCAL_I915_EXEC_HANDLE_LUT;
+	execbuf.flags |= LOCAL_I915_EXEC_NO_RELOC;
+
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf)) {
+		len = gem_linear_blt(fd, buf, src, dst, object_size, reloc);
+		igt_assert(len == execbuf.batch_len);
+		gem_write(fd, handle, 0, buf, len);
+		execbuf.flags = ring;
+		do_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf);
+	}
+	gem_sync(fd, handle);
 
 	for (count = 1; count <= 1<<12; count <<= 1) {
 		struct timeval start, end;
+		const int reps = 9;
+		double t[reps], sum;
+		int n;
 
-		gettimeofday(&start, NULL);
-		for (int loop = 0; loop < count; loop++)
-			gem_execbuf(fd, &execbuf);
-		gem_sync(fd, handle);
-		gettimeofday(&end, NULL);
+		for (n = 0; n < reps; n++) {
+			gettimeofday(&start, NULL);
+			for (int loop = 0; loop < count; loop++)
+				gem_execbuf(fd, &execbuf);
+			gem_sync(fd, handle);
+			gettimeofday(&end, NULL);
+			t[n] = elapsed(&start, &end, count);
+		}
+		qsort(t, n, sizeof(double), dcmp);
+		sum = 0;
+		for (n = 2; n < reps - 2; n++)
+			sum += t[n];
+		sum /= reps - 4;
 		igt_info("Time to blt %d bytes x %6d:	%7.3fµs, %s\n",
-			 object_size, count,
-			 elapsed(&start, &end, count),
-			 bytes_per_sec((char *)buf, object_size/elapsed(&start, &end, count)*1e6));
+			 object_size, count, sum,
+			 bytes_per_sec((char *)buf, object_size/sum*1e6));
 		fflush(stdout);
 	}
 	gem_close(fd, handle);
@@ -253,11 +264,95 @@ static void run(int object_size)
 	close(fd);
 }
 
+static int sysfs_read(const char *name)
+{
+	char buf[4096];
+	int sysfd;
+	int len;
+
+	sprintf(buf, "/sys/class/drm/card%d/%s",
+		drm_get_card(), name);
+	sysfd = open(buf, O_RDONLY);
+	if (sysfd < 0)
+		return -1;
+
+	len = read(sysfd, buf, sizeof(buf)-1);
+	close(sysfd);
+	if (len < 0)
+		return -1;
+
+	buf[len] = '\0';
+	return atoi(buf);
+}
+
+static int sysfs_write(const char *name, int value)
+{
+	char buf[4096];
+	int sysfd;
+	int len;
+
+	sprintf(buf, "/sys/class/drm/card%d/%s",
+		drm_get_card(), name);
+	sysfd = open(buf, O_WRONLY);
+	if (sysfd < 0)
+		return -1;
+
+	len = sprintf(buf, "%d", value);
+	len = write(sysfd, buf, len);
+	close(sysfd);
+
+	if (len < 0)
+		return len;
+
+	return 0;
+}
+
+static void set_auto_freq(void)
+{
+	int min = sysfs_read("gt_RPn_freq_mhz");
+	int max = sysfs_read("gt_RP0_freq_mhz");
+	if (max <= min)
+		return;
+
+	igt_debug("Setting min to %dMHz, and max to %dMHz\n", min, max);
+	sysfs_write("gt_min_freq_mhz", min);
+	sysfs_write("gt_max_freq_mhz", max);
+}
+
+static void set_min_freq(void)
+{
+	int min = sysfs_read("gt_RPn_freq_mhz");
+	igt_require(min > 0);
+	igt_debug("Setting min/max to %dMHz\n", min);
+	igt_require(sysfs_write("gt_min_freq_mhz", min) == 0 &&
+		    sysfs_write("gt_max_freq_mhz", min) == 0);
+}
+
+static void set_max_freq(void)
+{
+	int max = sysfs_read("gt_RP0_freq_mhz");
+	igt_require(max > 0);
+	igt_debug("Setting min/max to %dMHz\n", max);
+	igt_require(sysfs_write("gt_max_freq_mhz", max) == 0 &&
+		    sysfs_write("gt_min_freq_mhz", max) == 0);
+}
+
+
 int main(int argc, char **argv)
 {
+	const struct {
+		const char *suffix;
+		void (*func)(void);
+	} rps[] = {
+		{ "", set_auto_freq },
+		{ "-min", set_min_freq },
+		{ "-max", set_max_freq },
+		{ NULL, NULL },
+	}, *r;
+	int min = -1, max = -1;
 	int i;
 
-	igt_simple_init(argc, argv);
+	igt_subtest_init(argc, argv);
 
 	igt_skip_on_simulation();
 
@@ -265,10 +360,35 @@ int main(int argc, char **argv)
 		for (i = 1; i < argc; i++) {
 			int object_size = atoi(argv[i]);
 			if (object_size)
-				run((object_size + 3) & -4);
+				run((object_size + 3) & -4, false);
 		}
-	} else
-		run(OBJECT_SIZE);
+		_exit(0); /* blergh */
+	}
 
-	return 0;
+	igt_fixture {
+		min = sysfs_read("gt_min_freq_mhz");
+		max = sysfs_read("gt_max_freq_mhz");
+	}
+
+	for (r = rps; r->suffix; r++) {
+		igt_fixture r->func();
+
+		igt_subtest_f("cold%s", r->suffix)
+			run(OBJECT_SIZE, false);
+
+		igt_subtest_f("normal%s", r->suffix)
+			run(OBJECT_SIZE, false);
+
+		igt_subtest_f("dumb-buf%s", r->suffix)
+			run(OBJECT_SIZE, true);
+	}
+
+	igt_fixture {
+		if (min > 0)
+			sysfs_write("gt_min_freq_mhz", min);
+		if (max > 0)
+			sysfs_write("gt_max_freq_mhz", max);
+	}
+
+	igt_exit();
 }
