@@ -230,6 +230,59 @@ test_write(int fd)
 	munmap(src, OBJECT_SIZE);
 }
 
+static void
+test_wc(int fd)
+{
+	unsigned long gtt_reads, gtt_writes, cpu_writes;
+	uint32_t handle;
+	void *gtt, *cpu;
+
+	handle = gem_create(fd, 4096);
+	cpu = gem_mmap__cpu(fd, handle, 0, 4096, PROT_READ | PROT_WRITE);
+	gem_set_domain(fd, handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
+	gem_close(fd, handle);
+
+	handle = gem_create(fd, 4096);
+	gtt = gem_mmap__gtt(fd, handle, 4096, PROT_READ | PROT_WRITE);
+	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	gem_close(fd, handle);
+
+	gtt_reads = 0;
+	igt_for_milliseconds(200) {
+		memcpy(cpu, gtt, 4096);
+		gtt_reads++;
+	}
+	igt_debug("%lu GTT reads in 200us\n", gtt_reads);
+
+	gtt_writes = 0;
+	igt_for_milliseconds(200) {
+		memcpy(gtt, cpu, 4096);
+		gtt_writes++;
+	}
+	igt_debug("%lu GTT writes in 200us\n", gtt_writes);
+
+	if (igt_setup_clflush()) {
+		cpu_writes = 0;
+		igt_for_milliseconds(200) {
+			igt_clflush_range(cpu, 4096);
+			cpu_writes++;
+		}
+		igt_debug("%lu CPU writes in 200us\n", cpu_writes);
+	} else
+		cpu_writes = gtt_writes;
+
+	munmap(cpu, 4096);
+	munmap(gtt, 4096);
+
+	igt_assert_f(gtt_writes > 2*gtt_reads,
+		     "Write-Combined writes are expected to be much faster than reads: read=%.2fMiB/s, write=%.2fMiB/s\n",
+		     5*gtt_reads/256., 5*gtt_writes/256.);
+
+	igt_assert_f(gtt_writes > cpu_writes/2,
+		     "Write-Combined writes are expected to be roughly equivalent to WB writes: WC (gtt)=%.2fMiB/s, WB (cpu)=%.2fMiB/s\n",
+		     5*gtt_writes/256., 5*cpu_writes/256.);
+}
+
 static void
 test_write_gtt(int fd)
 {
@@ -280,7 +333,7 @@ test_coherency(int fd)
 	gem_close(fd, handle);
 }
 
-static int tile_width(uint32_t devid, int tiling)
+static int min_tile_width(uint32_t devid, int tiling)
 {
 	if (intel_gen(devid) == 2)
 		return 128;
@@ -292,6 +345,16 @@ static int tile_width(uint32_t devid, int tiling)
 		return 128;
 }
 
+static int max_tile_width(uint32_t devid, int tiling)
+{
+	if (intel_gen(devid) >= 7)
+		return 256 << 10;
+	else if (intel_gen(devid) >= 4)
+		return 128 << 10;
+	else
+		return 8 << 10;
+}
+
 static void
 test_huge_bo(int fd, int huge, int tiling)
 {
@@ -301,7 +364,7 @@ test_huge_bo(int fd, int huge, int tiling)
 	char *linear_pattern;
 	uint64_t size, last_offset;
 	uint32_t devid = intel_get_drm_devid(fd);
-	int pitch = tile_width(devid, tiling);
+	int pitch = min_tile_width(devid, tiling);
 	int i;
 
 	switch (huge) {
@@ -410,7 +473,7 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
 
 	bo = gem_create(fd, huge_object_size);
 	if (tiling_a)
-		igt_require(__gem_set_tiling(fd, bo, tiling_a, tile_width(devid, tiling_a)) == 0);
+		igt_require(__gem_set_tiling(fd, bo, tiling_a, min_tile_width(devid, tiling_a)) == 0);
 	a = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
 	igt_require(a);
 	gem_close(fd, bo);
@@ -422,7 +485,7 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
 
 	bo = gem_create(fd, huge_object_size);
 	if (tiling_b)
-		igt_require(__gem_set_tiling(fd, bo, tiling_b, tile_width(devid, tiling_b)) == 0);
+		igt_require(__gem_set_tiling(fd, bo, tiling_b, max_tile_width(devid, tiling_b)) == 0);
 	b = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
 	igt_require(b);
 	gem_close(fd, bo);
@@ -600,6 +663,8 @@ igt_main
 		run_without_prefault(fd, test_write_gtt);
 	igt_subtest("basic-write-cpu-read-gtt")
 		test_write_cpu_read_gtt(fd);
+	igt_subtest("basic-wc")
+		test_wc(fd);
 
 	igt_subtest("basic-small-bo")
 		test_huge_bo(fd, -1, I915_TILING_NONE);
