@@ -74,11 +74,11 @@ static void scratch_buf_init(struct igt_buf *buf,
 
 static void scratch_buf_fini(struct igt_buf *buf)
 {
-	dri_bo_unreference(buf->bo);
+	drm_intel_bo_unreference(buf->bo);
 	memset(buf, 0, sizeof(*buf));
 }
 
-static void fork_rcs_copy(int target, dri_bo **dst, int count, unsigned flags)
+static void fork_rcs_copy(int target, drm_intel_bo **dst, int count, unsigned flags)
 #define CREATE_CONTEXT 0x1
 {
 	igt_render_copyfunc_t render_copy;
@@ -143,7 +143,7 @@ static void fork_rcs_copy(int target, dri_bo **dst, int count, unsigned flags)
 	}
 }
 
-static void fork_bcs_copy(int target, dri_bo **dst, int count)
+static void fork_bcs_copy(int target, drm_intel_bo **dst, int count)
 {
 	int devid;
 
@@ -167,7 +167,7 @@ static void fork_bcs_copy(int target, dri_bo **dst, int count)
 		igt_assert(batch);
 
 		for (int i = 0; i <= target; i++) {
-			dri_bo *src[2];
+			drm_intel_bo *src[2];
 
 			src[0] = create_bo(dst[child]->bufmgr,
 					   ~0);
@@ -177,13 +177,13 @@ static void fork_bcs_copy(int target, dri_bo **dst, int count)
 			intel_copy_bo(batch, src[0], src[1], SIZE);
 			intel_copy_bo(batch, dst[child], src[0], SIZE);
 
-			dri_bo_unreference(src[1]);
-			dri_bo_unreference(src[0]);
+			drm_intel_bo_unreference(src[1]);
+			drm_intel_bo_unreference(src[0]);
 		}
 	}
 }
 
-static void surfaces_check(dri_bo **bo, int count, uint32_t expected)
+static void surfaces_check(drm_intel_bo **bo, int count, uint32_t expected)
 {
 	for (int child = 0; child < count; child++) {
 		uint32_t *ptr;
@@ -236,6 +236,8 @@ static void flink_and_close(void)
 	gem_sync(fd2, flinked_bo);
 	gem_close(fd2, flinked_bo);
 
+	igt_drop_caches_set(DROP_RETIRE);
+
 	/* the flinked bo VMA should have been cleared now, so a new bo of the
 	 * same size should get the same offset
 	 */
@@ -252,52 +254,41 @@ static void flink_and_close(void)
 
 static void flink_and_exit(void)
 {
-	uint32_t fd, fd2;
+	uint32_t fd, fd2, fd3;
 	uint32_t bo, flinked_bo, name;
-	char match[100];
-	int to_match;
-	bool matched;
-	int retry = 0;
-	const int retries = 50;
+	char match[20];
 
 	fd = drm_open_driver(DRIVER_INTEL);
 	igt_require(gem_uses_full_ppgtt(fd));
 
 	bo = gem_create(fd, 4096);
 	name = gem_flink(fd, bo);
-
-	to_match  = snprintf(match, sizeof(match), "(name: %u)", name);
-	igt_assert(to_match < sizeof(match));
+	snprintf(match, sizeof(match), "(name: %u)", name);
 
 	fd2 = drm_open_driver(DRIVER_INTEL);
-
 	flinked_bo = gem_open(fd2, name);
+
+	/* Verify VMA is not there yet. */
+	igt_assert(!igt_debugfs_search("i915_gem_gtt", match));
+
 	exec_and_get_offset(fd2, flinked_bo);
-	gem_sync(fd2, flinked_bo);
 
-	/* Verify looking for string works OK. */
-	matched = igt_debugfs_search("i915_gem_gtt", match);
-	igt_assert_eq(matched, true);
-
-	gem_close(fd2, flinked_bo);
+	/* Verify VMA has been created. */
+	igt_assert(igt_debugfs_search("i915_gem_gtt", match));
 
 	/* Close the context. */
 	close(fd2);
 
-retry:
-	/* Give cleanup some time to run. */
-	usleep(100000);
-
-	/* The flinked bo VMA should have been cleared now, so list of VMAs
-	 * in debugfs should not contain the one for the imported object.
+	/* Execute a different and unrelated (wrt object sharing) context to
+	 * ensure engine drops its last context reference.
 	 */
-	matched = igt_debugfs_search("i915_gem_gtt", match);
-	if (matched && retry++ < retries)
-		goto retry;
+	fd3 = drm_open_driver(DRIVER_INTEL);
+	exec_and_get_offset(fd3, gem_create(fd3, 4096));
+	close(fd3);
 
-	igt_assert_eq(matched, false);
+	igt_drop_caches_set(DROP_ACTIVE | DROP_RETIRE);
+	igt_assert(!igt_debugfs_search("i915_gem_gtt", match));
 
-	gem_close(fd, bo);
 	close(fd);
 }
 
@@ -307,7 +298,7 @@ int main(int argc, char **argv)
 	igt_subtest_init(argc, argv);
 
 	igt_subtest("blt-vs-render-ctx0") {
-		dri_bo *bcs[1], *rcs[N_CHILD];
+		drm_intel_bo *bcs[1], *rcs[N_CHILD];
 
 		fork_bcs_copy(0x4000, bcs, 1);
 		fork_rcs_copy(0x8000 / N_CHILD, rcs, N_CHILD, 0);
@@ -319,7 +310,7 @@ int main(int argc, char **argv)
 	}
 
 	igt_subtest("blt-vs-render-ctxN") {
-		dri_bo *bcs[1], *rcs[N_CHILD];
+		drm_intel_bo *bcs[1], *rcs[N_CHILD];
 
 		fork_rcs_copy(0x8000 / N_CHILD, rcs, N_CHILD, CREATE_CONTEXT);
 		fork_bcs_copy(0x4000, bcs, 1);
