@@ -29,6 +29,7 @@
  * workload for the 3D engine.
  */
 
+#include "igt.h"
 #include <stdbool.h>
 #include <unistd.h>
 #include <cairo.h>
@@ -44,13 +45,7 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION("Basic test for the render_copy() function.");
 
@@ -145,7 +140,7 @@ int main(int argc, char **argv)
 				   opt_handler, NULL);
 
 	igt_fixture {
-		data.drm_fd = drm_open_any_render();
+		data.drm_fd = drm_open_driver_render(DRIVER_INTEL);
 		data.devid = intel_get_drm_devid(data.drm_fd);
 
 		data.bufmgr = drm_intel_bufmgr_gem_init(data.drm_fd, 4096);
