@@ -34,6 +34,14 @@
 #include "igt.h"
 #include "igt_gt.h"
 
+#define INTERRUPTIBLE 0x1
+#define HANG 0x2
+#define CHILD 0x8
+#define FORKED 0x8
+#define BOMB 0x10
+#define SUSPEND 0x20
+#define HIBERNATE 0x40
+
 static void check_bo(int fd, uint32_t handle)
 {
 	uint32_t *map;
@@ -47,10 +55,10 @@ static void check_bo(int fd, uint32_t handle)
 	munmap(map, 4096);
 }
 
-static void fill_ring(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
+static void fill_ring(int fd,
+		      struct drm_i915_gem_execbuffer2 *execbuf,
+		      unsigned flags)
 {
-	int i;
-
 	/* The ring we've been using is 128k, and each rendering op
 	 * will use at least 8 dwords:
 	 *
@@ -67,16 +75,12 @@ static void fill_ring(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 	 * doing this, we aren't likely to with this test.
 	 */
 	igt_debug("Executing execbuf %d times\n", 128*1024/(8*4));
-	for (i = 0; i < 128*1024 / (8 * 4); i++)
-		gem_execbuf(fd, execbuf);
+	igt_while_interruptible(flags & INTERRUPTIBLE) {
+		for (int i = 0; i < 128*1024 / (8 * 4); i++)
+			gem_execbuf(fd, execbuf);
+	}
 }
 
-#define INTERRUPTIBLE 0x1
-#define HANG 0x2
-#define CHILD 0x8
-#define FORKED 0x8
-#define BOMB 0x10
-
 static void run_test(int fd, unsigned ring, unsigned flags)
 {
 	const int gen = intel_gen(intel_get_drm_devid(fd));
@@ -92,21 +96,25 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 	igt_skip_on_f(gen == 6 && (ring & ~(3<<13)) == I915_EXEC_BSD,
 		      "MI_STORE_DATA broken on gen6 bsd\n");
 
+	if (flags & (SUSPEND | HIBERNATE))
+		run_test(fd, ring, 0);
+
 	gem_quiescent_gpu(fd);
 
 	memset(&execbuf, 0, sizeof(execbuf));
 	execbuf.buffers_ptr = (uintptr_t)obj;
-	execbuf.buffer_count = 2;
-	execbuf.flags = ring | (1 << 11);
+	execbuf.flags = ring | (1 << 11) | (1 << 12);
 	if (gen < 6)
 		execbuf.flags |= I915_EXEC_SECURE;
 
 	memset(obj, 0, sizeof(obj));
 	obj[0].handle = gem_create(fd, 4096);
+	gem_write(fd, obj[0].handle, 0, &bbe, sizeof(bbe));
+	execbuf.buffer_count = 1;
+	igt_require(__gem_execbuf(fd, &execbuf) == 0);
+
 	obj[0].flags |= EXEC_OBJECT_WRITE;
 	obj[1].handle = gem_create(fd, 1024*16 + 4096);
-	gem_write(fd, obj[1].handle, 0, &bbe, sizeof(bbe));
-	igt_require(__gem_execbuf(fd, &execbuf) == 0);
 
 	obj[1].relocs_ptr = (uintptr_t)reloc;
 	obj[1].relocation_count = 1024;
@@ -121,7 +129,6 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 	for (i = 0; i < 1024; i++) {
 		uint64_t offset;
 
-		reloc[i].target_handle = obj[0].handle;
 		reloc[i].presumed_offset = obj[0].offset;
 		reloc[i].offset = (b - batch + 1) * sizeof(*batch);
 		reloc[i].delta = i * sizeof(uint32_t);
@@ -145,6 +152,7 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 	}
 	*b++ = MI_BATCH_BUFFER_END;
 	munmap(batch, 16*1024+4096);
+	execbuf.buffer_count = 2;
 	gem_execbuf(fd, &execbuf);
 	check_bo(fd, obj[0].handle);
 
@@ -152,29 +160,29 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 	if (flags & HANG)
 		hang = igt_hang_ring(fd, ring & ~(3<<13));
 
-	if (flags & INTERRUPTIBLE)
-		igt_fork_signal_helper();
-
 	if (flags & (CHILD | FORKED | BOMB)) {
 		int nchild;
 
-		if (flags & CHILD)
-			nchild = 1;
-		else if (flags & FORKED)
+		if (flags & FORKED)
 			nchild = sysconf(_SC_NPROCESSORS_ONLN);
-		else
+		else if (flags & BOMB)
 			nchild = 8*sysconf(_SC_NPROCESSORS_ONLN);
+		else
+			nchild = 1;
 
 		igt_debug("Forking %d children\n", nchild);
 		igt_fork(child, nchild)
-			fill_ring(fd, &execbuf);
+			fill_ring(fd, &execbuf, flags);
+
+		if (flags & SUSPEND)
+			igt_system_suspend_autoresume();
+
+		if (flags & HIBERNATE)
+			igt_system_hibernate_autoresume();
 
 		igt_waitchildren();
 	} else
-		fill_ring(fd, &execbuf);
-
-	if (flags & INTERRUPTIBLE)
-		igt_stop_signal_helper();
+		fill_ring(fd, &execbuf, flags);
 
 	if (flags & HANG)
 		igt_post_hang_ring(fd, hang);
@@ -183,6 +191,11 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 
 	gem_close(fd, obj[1].handle);
 	gem_close(fd, obj[0].handle);
+
+	gem_quiescent_gpu(fd);
+
+	if (flags & (SUSPEND | HIBERNATE))
+		run_test(fd, ring, 0);
 }
 
 igt_main
@@ -190,28 +203,31 @@ igt_main
 	const struct {
 		const char *suffix;
 		unsigned flags;
+		bool basic;
 	} modes[] = {
-		{ "", 0 },
-		{ "-interruptible", INTERRUPTIBLE },
-		{ "-hang", HANG },
+		{ "", 0, true},
+		{ "-interruptible", INTERRUPTIBLE, true },
+		{ "-hang", HANG, true },
 		{ "-child", CHILD },
-		{ "-forked", FORKED },
+		{ "-forked", FORKED, true },
 		{ "-bomb", BOMB | INTERRUPTIBLE },
-		{ NULL, 0 }
-	}, *mode;
+		{ "-S3", BOMB | SUSPEND },
+		{ "-S4", BOMB | HIBERNATE },
+		{ NULL }
+	}, *m;
 	const struct intel_execution_engine *e;
 	int fd;
 
 	igt_fixture
 		fd = drm_open_driver_master(DRIVER_INTEL);
 
-	for (mode = modes; mode->suffix; mode++) {
+	for (m = modes; m->suffix; m++) {
 		for (e = intel_execution_engines; e->name; e++) {
 			igt_subtest_f("%s%s%s",
-				      e->exec_id ? "" : "basic-",
+				      m->basic && !e->exec_id ? "basic-" : "",
 				      e->name,
-				      mode->suffix)
-				run_test(fd, e->exec_id | e->flags, mode->flags);
+				      m->suffix)
+				run_test(fd, e->exec_id | e->flags, m->flags);
 		}
 	}
 
