@@ -720,10 +720,10 @@ static void atomic_setup(struct kms_atomic_state *state)
 	drmModePlaneResPtr res_plane;
 	int i;
 
-	desc->fd = drm_open_driver_master(DRIVER_INTEL);
+	desc->fd = drm_open_driver_master(DRIVER_ANY);
 	igt_assert_fd(desc->fd);
 
-	do_or_die(drmSetClientCap(desc->fd, DRM_CLIENT_CAP_ATOMIC, 1));
+	igt_skip_on(drmSetClientCap(desc->fd, DRM_CLIENT_CAP_ATOMIC, 1));
 
 	res = drmModeGetResources(desc->fd);
 	res_plane = drmModeGetPlaneResources(desc->fd);
@@ -1361,7 +1361,7 @@ igt_main
 
 	igt_subtest("atomic_invalid_params") {
 		struct kms_atomic_state *scratch = atomic_state_dup(current);
-		struct kms_atomic_crtc_state *crtc = &scratch->crtcs[0];
+		struct kms_atomic_crtc_state *crtc = find_crtc(scratch, true);
 		struct kms_atomic_plane_state *plane =
 			find_plane(scratch, NUM_PLANE_TYPE_PROPS, crtc);
 		struct kms_atomic_connector_state *conn =
