@@ -24,16 +24,13 @@
 
 #include "igt.h"
 #include <math.h>
-
+#include <sys/stat.h>
 
 IGT_TEST_DESCRIPTION("Test display panel fitting");
 
 typedef struct {
-	uint32_t devid;
 	int drm_fd;
 	igt_display_t display;
-	igt_crc_t ref_crc;
-	igt_pipe_crc_t *pipe_crc;
 
 	int image_w;
 	int image_h;
@@ -60,10 +57,6 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 
 	igt_output_set_pipe(output, pipe);
 
-	/* create the pipe_crc object for this pipe */
-	igt_pipe_crc_free(data->pipe_crc);
-	data->pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-
 	/* before allocating, free if any older fb */
 	if (data->fb_id1) {
 		igt_remove_fb(data->drm_fd, &data->fb1);
@@ -72,10 +65,10 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 
 	/* allocate fb for plane 1 */
 	data->fb_id1 = igt_create_pattern_fb(data->drm_fd,
-					     mode->hdisplay, mode->vdisplay,
-					     DRM_FORMAT_XRGB8888,
-					     LOCAL_I915_FORMAT_MOD_X_TILED, /* tiled */
-					     &data->fb1);
+						mode->hdisplay, mode->vdisplay,
+						DRM_FORMAT_XRGB8888,
+						LOCAL_DRM_FORMAT_MOD_NONE,
+						&data->fb1);
 	igt_assert(data->fb_id1);
 
 	/*
@@ -94,9 +87,9 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 	if (s == COMMIT_LEGACY) {
 		int ret;
 		ret = drmModeSetCrtc(data->drm_fd,
-				output->config.crtc->crtc_id,
+				plane->pipe->crtc_id,
 				data->fb_id1,
-				plane->pan_x, plane->pan_y,
+				plane->src_x, plane->src_y,
 				&output->id,
 				1,
 				mode);
@@ -110,9 +103,6 @@ static void cleanup_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
 {
 	igt_display_t *display = &data->display;
 
-	igt_pipe_crc_free(data->pipe_crc);
-	data->pipe_crc = NULL;
-
 	if (data->fb_id1) {
 		igt_remove_fb(data->drm_fd, &data->fb1);
 		data->fb_id1 = 0;
@@ -147,13 +137,22 @@ static void test_panel_fitting(data_t *d)
 	enum pipe pipe;
 	int valid_tests = 0;
 
-	for_each_connected_output(display, output) {
+	for_each_pipe_with_valid_output(display, pipe, output) {
 		drmModeModeInfo *mode, native_mode;
+		bool scaling_mode_set;
 
-		igt_require(output->config.connector->connector_type ==
-			DRM_MODE_CONNECTOR_eDP);
+		scaling_mode_set = kmstest_get_property(d->drm_fd,
+			output->config.connector->connector_id,
+			DRM_MODE_OBJECT_CONNECTOR,
+			"scaling mode",
+			NULL,
+			NULL,
+			NULL);
+
+		/* Check that the "scaling mode" property has been set. */
+		if (!scaling_mode_set)
+			continue;
 
-		pipe = output->config.pipe;
 		igt_output_set_pipe(output, pipe);
 
 		mode = igt_output_get_mode(output);
@@ -168,7 +167,7 @@ static void test_panel_fitting(data_t *d)
 
 		d->fb_id2 = igt_create_image_fb(d->drm_fd, 0, 0,
 						DRM_FORMAT_XRGB8888,
-						LOCAL_I915_FORMAT_MOD_X_TILED, /* tiled */
+						LOCAL_DRM_FORMAT_MOD_NONE,
 						FILE_NAME, &d->fb2);
 		igt_assert(d->fb_id2);
 
@@ -215,18 +214,90 @@ static void test_panel_fitting(data_t *d)
 	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
 }
 
-igt_simple_main
+static void
+test_panel_fitting_fastset(igt_display_t *display, const enum pipe pipe, igt_output_t *output)
+{
+	igt_plane_t *primary, *sprite;
+	drmModeModeInfo mode;
+	struct igt_fb black, red;
+
+	igt_assert(kmstest_get_connector_default_mode(display->drm_fd, output->config.connector, &mode));
+
+	igt_output_override_mode(output, &mode);
+	igt_output_set_pipe(output, pipe);
+
+	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+	sprite = igt_output_get_plane(output, IGT_PLANE_2);
+
+	igt_create_color_fb(display->drm_fd, mode.hdisplay, mode.vdisplay,
+			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+			    0.f, 0.f, 0.f, &black);
+
+	igt_create_color_fb(display->drm_fd, 640, 480,
+			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+			    1.f, 0.f, 0.f, &red);
+
+	igt_plane_set_fb(primary, &black);
+	igt_plane_set_fb(sprite, &red);
+
+	igt_display_commit2(display, COMMIT_ATOMIC);
+
+	mode.hdisplay = 640;
+	mode.vdisplay = 480;
+	igt_output_override_mode(output, &mode);
+
+	igt_plane_set_fb(sprite, NULL);
+	igt_plane_set_fb(primary, &red);
+
+	/* Don't pass ALLOW_MODESET with overridden mode, force fastset. */
+	igt_display_commit_atomic(display, 0, NULL);
+
+	igt_plane_set_fb(primary, NULL);
+	igt_output_set_pipe(output, PIPE_NONE);
+	igt_output_override_mode(output, NULL);
+}
+
+static void test_atomic_fastset(igt_display_t *display)
+{
+	igt_output_t *output;
+	enum pipe pipe;
+	int valid_tests = 0;
+	struct stat sb;
+
+	/* Until this is force enabled, force modeset evasion. */
+	if (stat("/sys/module/i915/parameters/fastboot", &sb) == 0)
+		igt_set_module_param_int("fastboot", 1);
+
+	igt_require(display->is_atomic);
+	igt_require(intel_gen(intel_get_drm_devid(display->drm_fd)) >= 5);
+
+	for_each_pipe_with_valid_output(display, pipe, output) {
+		if (!output->config.atomic_props_connector[IGT_CONNECTOR_SCALING_MODE])
+			continue;
+
+		test_panel_fitting_fastset(display, pipe, output);
+		valid_tests++;
+	}
+	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
+}
+
+igt_main
 {
 	data_t data = {};
 
-	igt_skip_on_simulation();
+	igt_fixture {
+		igt_skip_on_simulation();
 
-	data.drm_fd = drm_open_driver(DRIVER_INTEL);
-	igt_require_pipe_crc();
-	igt_display_init(&data.display, data.drm_fd);
-	data.devid = intel_get_drm_devid(data.drm_fd);
+		data.drm_fd = drm_open_driver(DRIVER_ANY);
+		igt_display_init(&data.display, data.drm_fd);
+	}
 
-	test_panel_fitting(&data);
+	igt_subtest("legacy")
+		test_panel_fitting(&data);
 
-	igt_display_fini(&data.display);
+	igt_subtest("atomic-fastset")
+		test_atomic_fastset(&data.display);
+
+	igt_fixture
+		igt_display_fini(&data.display);
 }
