@@ -112,7 +112,6 @@ struct connector {
 	drmModeEncoder *encoder;
 	drmModeConnector *connector;
 	int crtc;
-	int crtc_idx;
 	int pipe;
 };
 
@@ -211,7 +210,6 @@ static void connector_find_preferred_mode(uint32_t connector_id,
 	c->connector = config.connector;
 	c->encoder = config.encoder;
 	c->crtc = config.crtc->crtc_id;
-	c->crtc_idx = config.crtc_idx;
 	c->pipe = config.pipe;
 
 	if (mode_num != -1) {
@@ -226,21 +224,15 @@ static void connector_find_preferred_mode(uint32_t connector_id,
 static void
 paint_color_key(struct igt_fb *fb_info)
 {
-	int i, j;
-	uint32_t *fb_ptr;
+	cairo_t *cr = igt_get_cairo_ctx(drm_fd, fb_info);
 
-	fb_ptr = gem_mmap__gtt(drm_fd, fb_info->gem_handle, fb_info->size,
-			       PROT_READ | PROT_WRITE);
+	cairo_rectangle(cr, crtc_x, crtc_y, crtc_w, crtc_h);
+	cairo_set_source_rgb(cr, .8, .8, .8);
+	cairo_fill(cr);
 
-	for (i = crtc_y; i < crtc_y + crtc_h; i++)
-		for (j = crtc_x; j < crtc_x + crtc_w; j++) {
-			uint32_t offset;
+	igt_assert(!cairo_status(cr));
 
-			offset = (i * fb_info->stride / 4) + j;
-			fb_ptr[offset] = SPRITE_COLOR_KEY;
-		}
-
-	munmap(fb_ptr, fb_info->size);
+	cairo_destroy(cr);
 }
 
 static void paint_image(cairo_t *cr, const char *file)
@@ -503,7 +495,7 @@ int update_display(bool probe)
 
 			if (test_preferred_mode || force_mode ||
 			    specified_mode_num != -1)
-				crtc_idx_mask &= ~(1 << connector->crtc_idx);
+				crtc_idx_mask &= ~(1 << connector->pipe);
 
 		}
 	}
@@ -730,7 +722,7 @@ int main(int argc, char **argv)
 	    specified_mode_num == -1 && !test_stereo_modes)
 		test_all_modes = 1;
 
-	drm_fd = drm_open_driver(DRIVER_INTEL);
+	drm_fd = drm_open_driver(DRIVER_ANY);
 
 	if (test_stereo_modes &&
 	    drmSetClientCap(drm_fd, DRM_CLIENT_CAP_STEREO_3D, 1) < 0) {
