@@ -151,6 +151,7 @@ static void dump_general_features(struct context *context,
 	printf("\tMessage: %s\n", YESNO(features->msg_enable));
 	printf("\tClear screen: %d\n", features->clear_screen);
 	printf("\tDVO color flip required: %s\n", YESNO(features->color_flip));
+
 	printf("\tExternal VBT: %s\n", YESNO(features->download_ext_vbt));
 	printf("\tEnable SSC: %s\n", YESNO(features->enable_ssc));
 	if (features->enable_ssc) {
@@ -171,13 +172,45 @@ static void dump_general_features(struct context *context,
 	       YESNO(features->enable_lfp_on_override));
 	printf("\tDisable SSC on clone: %s\n",
 	       YESNO(features->disable_ssc_ddt));
+	printf("\tUnderscan support for VGA timings: %s\n",
+	       YESNO(features->underscan_vga_timings));
+	if (context->bdb->version >= 183)
+		printf("\tDynamic CD clock: %s\n", YESNO(features->dynamic_cdclk));
+	printf("\tHotplug support in VBIOS: %s\n",
+	       YESNO(features->vbios_hotplug_support));
+
 	printf("\tDisable smooth vision: %s\n",
 	       YESNO(features->disable_smooth_vision));
 	printf("\tSingle DVI for CRT/DVI: %s\n", YESNO(features->single_dvi));
+	if (context->bdb->version >= 181)
+		printf("\tEnable 180 degree rotation: %s\n", YESNO(features->rotate_180));
+	printf("\tInverted FDI Rx polarity: %s\n", YESNO(features->fdi_rx_polarity));
+	if (context->bdb->version >= 160) {
+		printf("\tExtended VBIOS mode: %s\n", YESNO(features->vbios_extended_mode));
+		printf("\tCopy iLFP DTD to SDVO LVDS DTD: %s\n", YESNO(features->copy_ilfp_dtd_to_sdvo_lvds_dtd));
+		printf("\tBest fit panel timing algorithm: %s\n", YESNO(features->panel_best_fit_timing));
+		printf("\tIgnore strap state: %s\n", YESNO(features->ignore_strap_state));
+	}
+
 	printf("\tLegacy monitor detect: %s\n",
 	       YESNO(features->legacy_monitor_detect));
+
 	printf("\tIntegrated CRT: %s\n", YESNO(features->int_crt_support));
 	printf("\tIntegrated TV: %s\n", YESNO(features->int_tv_support));
+	printf("\tIntegrated EFP: %s\n", YESNO(features->int_efp_support));
+	printf("\tDP SSC enable: %s\n", YESNO(features->dp_ssc_enable));
+	if (features->dp_ssc_enable) {
+		if (IS_VALLEYVIEW(context->devid) || IS_CHERRYVIEW(context->devid) ||
+		    IS_BROXTON(context->devid))
+			printf("\tSSC frequency: 100 MHz\n");
+		else if (HAS_PCH_SPLIT(context->devid))
+			printf("\tSSC frequency: %s\n", features->dp_ssc_freq ?
+			       "100 MHz" : "120 MHz");
+		else
+			printf("\tSSC frequency: %s\n", features->dp_ssc_freq ?
+			       "100 MHz" : "96 MHz");
+	}
+	printf("\tDP SSC dongle supported: %s\n", YESNO(features->dp_ssc_dongle_supported));
 }
 
 static void dump_backlight_info(struct context *context,
@@ -361,8 +394,6 @@ static const char *efp_conn(uint8_t type)
 	return "unknown";
 }
 
-
-
 static void dump_child_device(struct context *context,
 			      struct child_device_config *child)
 {
@@ -390,15 +421,72 @@ static void dump_child_device(struct context *context,
 		printf("\t\tDevice type: 0x%04x (%s)\n", efp->device_type,
 		       child_device_type(efp->device_type));
 		dump_child_device_type_bits(efp->device_type);
-		printf("\t\tPort: 0x%02x (%s)\n", efp->port,
-		       efp_port(efp->port));
+		printf("\t\tI2C speed: 0x%02x\n", efp->i2c_speed);
+		printf("\t\tDP onboard redriver: 0x%02x\n", efp->dp_onboard_redriver);
+		printf("\t\tDP ondock redriver: 0x%02x\n", efp->dp_ondock_redriver);
+		printf("\t\tHDMI max data rate: 0x%02x\n", efp->hdmi_max_data_rate);
+		printf("\t\tHDMI level shifter value: 0x%02x\n", efp->hdmi_level_shifter_value);
+		printf("\t\tOffset to DTD buffer for edidless EFP: 0x%02x\n", efp->dtd_buf_ptr);
+		printf("\t\tDual pipe ganged eDP: %s\n", YESNO(efp->ganged_edp));
+		printf("\t\tCompression method CPS: %s\n", YESNO(efp->compression_method));
+		printf("\t\tCompression enable: %s\n", YESNO(efp->compression_enable));
+		printf("\t\tEdidless EFP: %s\n", YESNO(efp->edidless_efp));
+		printf("\t\tCompression structure index: 0x%02x)\n", efp->compression_structure_index);
+		printf("\t\tSlave DDI port: 0x%02x (%s)\n", efp->slave_port, efp_port(efp->slave_port));
+		printf("\t\tAIM offset: %d\n", child->addin_offset);
+		printf("\t\tPort: 0x%02x (%s)\n", efp->port, efp_port(efp->port));
+		printf("\t\tAIM I2C pin: 0x%02x\n", efp->i2c_pin);
+		printf("\t\tAIM Slave address: 0x%02x\n", efp->slave_addr);
 		printf("\t\tDDC pin: 0x%02x\n", efp->ddc_pin);
-		printf("\t\tDock port: 0x%02x (%s)\n", efp->docked_port,
-		       efp_port(efp->docked_port));
-		printf("\t\tHDMI compatible? %s\n", efp->hdmi_compat ? "Yes" : "No");
-		printf("\t\tInfo: %s\n", efp_conn(efp->conn_info));
+		printf("\t\tEDID buffer ptr: 0x%02x\n", efp->edid_ptr);
+		printf("\t\tDVO config: 0x%02x\n", efp->dvo_config);
+		printf("\t\tHPD sense invert: %s\n", YESNO(efp->hpd_invert));
+		printf("\t\tIboost enable: %s\n", YESNO(efp->iboost_enable));
+		printf("\t\tOnboard LSPCON: %s\n", YESNO(efp->onboard_lspcon));
+		printf("\t\tLane reversal: %s\n", YESNO(efp->lane_reversal));
+		printf("\t\tEFP routed through dock: %s\n", YESNO(efp->efp_docked_port));
+		printf("\t\tHDMI compatible? %s\n", YESNO(efp->hdmi_compat));
+		printf("\t\tDP compatible? %s\n", YESNO(efp->dp_compat));
+		printf("\t\tTMDS compatible? %s\n", YESNO(efp->tmds_compat));
 		printf("\t\tAux channel: 0x%02x\n", efp->aux_chan);
 		printf("\t\tDongle detect: 0x%02x\n", efp->dongle_detect);
+		printf("\t\tIntegrated encoder instead of SDVO: %s\n", YESNO(efp->integrated_encoder));
+		printf("\t\tHotplu connect status: 0x%02x\n", efp->hpd_status);
+		printf("\t\tSDVO stall signal available: %s\n", YESNO(efp->sdvo_stall));
+		printf("\t\tPipe capabilities: 0x%02x\n", efp->pipe_cap);
+		printf("\t\tDVO wiring: 0x%02x\n", efp->dvo_wiring);
+		printf("\t\tMIPI bridge type:");
+		switch (efp->mipi_bridge_type) {
+		case 1:
+			printf("ASUS\n");
+			break;
+		case 2:
+			printf("Toshiba\n");
+			break;
+		case 3:
+			printf("Renesas\n");
+			break;
+		default:
+			printf("(unknown value %d)\n", efp->mipi_bridge_type);
+			break;
+		}
+		printf("\t\tDevice class extendsion: 0x%02x\n", efp->device_class_ext);
+		printf("\t\tDVO function: 0x%02x\n", efp->dvo_function);
+	}
+
+	if (context->bdb->version >= 195) {
+		struct efp_child_device_config *efp =
+			(struct efp_child_device_config *)child;
+		printf("\t\tDP USB type C support: %s\n", YESNO(efp->dp_usb_type_c));
+		printf("\t\t2X DP GPIO index: 0x%02x\n", efp->dp_usb_type_c_2x_gpio_index);
+		printf("\t\t2X DP GPIO pin number: 0x%02x\n", efp->dp_usb_type_c_2x_gpio_pin);
+	}
+
+	if (context->bdb->version >= 196) {
+		struct efp_child_device_config *efp =
+			(struct efp_child_device_config *)child;
+		printf("\t\tIBoost level for HDMI: 0x%02x\n", efp->iboost_hdmi);
+		printf("\t\tIBoost level for DP/eDP: 0x%02x\n", efp->iboost_dp);
 	}
 }
 
@@ -666,14 +754,14 @@ static void dump_edp(struct context *context,
 		msa = (edp->sdrrs_msa_timing_delay >> (i * 2)) & 3;
 		printf("\t\teDP sDRRS MSA Delay: Lane %d\n", msa + 1);
 
-		printf("\t\tLink params:\n");
+		printf("\t\tFast link params:\n");
 		printf("\t\t\trate: ");
-		if (edp->link_params[i].rate == EDP_RATE_1_62)
+		if (edp->fast_link_params[i].rate == EDP_RATE_1_62)
 			printf("1.62G\n");
-		else if (edp->link_params[i].rate == EDP_RATE_2_7)
+		else if (edp->fast_link_params[i].rate == EDP_RATE_2_7)
 			printf("2.7G\n");
 		printf("\t\t\tlanes: ");
-		switch (edp->link_params[i].lanes) {
+		switch (edp->fast_link_params[i].lanes) {
 		case EDP_LANE_1:
 			printf("x1 mode\n");
 			break;
@@ -685,11 +773,11 @@ static void dump_edp(struct context *context,
 			break;
 		default:
 			printf("(unknown value %d)\n",
-			       edp->link_params[i].lanes);
+			       edp->fast_link_params[i].lanes);
 			break;
 		}
 		printf("\t\t\tpre-emphasis: ");
-		switch (edp->link_params[i].preemphasis) {
+		switch (edp->fast_link_params[i].preemphasis) {
 		case EDP_PREEMPHASIS_NONE:
 			printf("none\n");
 			break;
@@ -704,11 +792,11 @@ static void dump_edp(struct context *context,
 			break;
 		default:
 			printf("(unknown value %d)\n",
-			       edp->link_params[i].preemphasis);
+			       edp->fast_link_params[i].preemphasis);
 			break;
 		}
 		printf("\t\t\tvswing: ");
-		switch (edp->link_params[i].vswing) {
+		switch (edp->fast_link_params[i].vswing) {
 		case EDP_VSWING_0_4V:
 			printf("0.4V\n");
 			break;
@@ -723,9 +811,99 @@ static void dump_edp(struct context *context,
 			break;
 		default:
 			printf("(unknown value %d)\n",
-			       edp->link_params[i].vswing);
+			       edp->fast_link_params[i].vswing);
 			break;
 		}
+
+		if (context->bdb->version >= 162) {
+			bool val = (edp->s3d_feature >> i) & 1;
+			printf("\t\tStereo 3D feature: %s\n", YESNO(val));
+		}
+
+		if (context->bdb->version >= 165) {
+			bool val = (edp->t3_optimization >> i) & 1;
+			printf("\t\tT3 optimization: %s\n", YESNO(val));
+		}
+
+		if (context->bdb->version >= 173) {
+			int val = (edp->vswing_preemph_table_selection >> (i * 4)) & 0xf;
+
+			printf("\t\tVswing/preemphasis table selection: ");
+			switch (val) {
+			case 0:
+				printf("Low power (200 mV)\n");
+				break;
+			case 1:
+				printf("Default (400 mV)\n");
+				break;
+			default:
+				printf("(unknown value %d)\n", val);
+				break;
+			}
+		}
+
+		if (context->bdb->version >= 182) {
+			bool val = (edp->fast_link_training >> i) & 1;
+			printf("\t\tFast link training: %s\n", YESNO(val));
+		}
+
+		if (context->bdb->version >= 185) {
+			bool val = (edp->dpcd_600h_write_required >> i) & 1;
+			printf("\t\tDPCD 600h write required: %s\n", YESNO(val));
+		}
+
+		if (context->bdb->version >= 186) {
+			printf("\t\tPWM delays:\n"
+			       "\t\t\tPWM on to backlight enable: %d\n"
+			       "\t\t\tBacklight disable to PWM off: %d\n",
+			       edp->pwm_delays[i].pwm_on_to_backlight_enable,
+			       edp->pwm_delays[i].backlight_disable_to_pwm_off);
+		}
+
+		if (context->bdb->version >= 199) {
+			bool val = (edp->full_link_params_provided >> i) & 1;
+
+			printf("\t\tFull link params provided: %s\n", YESNO(val));
+			printf("\t\tFull link params:\n");
+			printf("\t\t\tpre-emphasis: ");
+			switch (edp->full_link_params[i].preemphasis) {
+			case EDP_PREEMPHASIS_NONE:
+				printf("none\n");
+				break;
+			case EDP_PREEMPHASIS_3_5dB:
+				printf("3.5dB\n");
+				break;
+			case EDP_PREEMPHASIS_6dB:
+				printf("6dB\n");
+				break;
+			case EDP_PREEMPHASIS_9_5dB:
+				printf("9.5dB\n");
+				break;
+			default:
+				printf("(unknown value %d)\n",
+				       edp->full_link_params[i].preemphasis);
+				break;
+			}
+			printf("\t\t\tvswing: ");
+			switch (edp->full_link_params[i].vswing) {
+			case EDP_VSWING_0_4V:
+				printf("0.4V\n");
+				break;
+			case EDP_VSWING_0_6V:
+				printf("0.6V\n");
+				break;
+			case EDP_VSWING_0_8V:
+				printf("0.8V\n");
+				break;
+			case EDP_VSWING_1_2V:
+				printf("1.2V\n");
+				break;
+			default:
+				printf("(unknown value %d)\n",
+				       edp->full_link_params[i].vswing);
+				break;
+			}
+		}
 	}
 }
 
